//-----------------------------------------------------------------------------
// File: CGameApp.cpp
//
// Desc: Game Application class, this is the central hub for all app processing
//
// Copyright (c) 1997-2002 Adam Hoult & Gary Simmons. All rights reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// CGameApp Specific Includes
//-----------------------------------------------------------------------------
#include "../../Source/MFC_Core/stdafx.h"
#include "../../resource.h"
#include "CGameApp.h"
#include "CCamera.h"


//-----------------------------------------------------------------------------
// CGameApp Member Functions
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Name : CGameApp () (Constructor)
// Desc : CGameApp Class Constructor
//-----------------------------------------------------------------------------
CGameApp::CGameApp()
{
	// Reset / Clear all required values
    m_hWnd          = NULL;
    m_pD3D          = NULL;
    m_pD3DDevice    = NULL;
    m_hIcon         = NULL;
    m_hMenu         = NULL;
    m_bLostDevice   = false;
    m_LastFrameRate = 0;
    
    // Set up initial states (these will be adjusted later if not supported)
    m_FillMode      = D3DFILL_SOLID;
    m_ColorOp       = D3DTOP_ADDSIGNED;
    m_MinFilter     = D3DTEXF_LINEAR;
    m_MagFilter     = D3DTEXF_LINEAR;
    m_MipFilter     = D3DTEXF_LINEAR;
    m_bSinglePass   = true;
    m_Anisotropy    = 1;

    // Clear arrays
    ZeroMemory( m_SkyTextures, 6 * sizeof(LPDIRECT3DTEXTURE9) );
}

//-----------------------------------------------------------------------------
// Name : ~CGameApp () (Destructor)
// Desc : CGameApp Class Destructor
//-----------------------------------------------------------------------------
CGameApp::~CGameApp()
{
	// Shut the engine down
    ShutDown();
}

//-----------------------------------------------------------------------------
// Name : InitInstance ()
// Desc : Initialises the entire Engine here.
//-----------------------------------------------------------------------------
bool CGameApp::InitInstance(HWND hwnd, CRect rc)
{
	this->m_hWnd = hwnd;
	m_hMenu = LoadMenu( (HINSTANCE)GetModuleHandle(NULL), MAKEINTRESOURCE( IDR_MAINFRAME ) );
	
	// Set menu only in windowed mode
    SetMenu( m_hWnd, m_hMenu );
   
    // Initialize Direct3D (Simple)
    if (!InitDirect3D(rc)) return false;
	
    // Build Objects
    if (!BuildObjects()) { ShutDown(); return false; }

    // Set up all required game states
    SetupGameState();

    // Setup our rendering environment
    SetupRenderStates();

    // Success!
	return true;
}

//-----------------------------------------------------------------------------
// Name : InitDirect3D () (Private)
// Desc : Performs a simple, non-enumerated, initialization of Direct3D
//-----------------------------------------------------------------------------
bool CGameApp::InitDirect3D(CRect rc)
{
    HRESULT               hRet;
    D3DPRESENT_PARAMETERS PresentParams;
    D3DCAPS9              Caps;
	D3DDISPLAYMODE        CurrentMode;
	
    // First of all create our D3D Object
    m_pD3D = Direct3DCreate9( D3D_SDK_VERSION );
    if (!m_pD3D) 
    {
        MessageBox( m_hWnd, _T("No compatible Direct3D object could be created."), _T("Fatal Error!"), MB_OK | MB_ICONSTOP | MB_APPLMODAL );
        return false;
    
    } // End if failure

    // Fill out a simple set of present parameters
    ZeroMemory( &PresentParams, sizeof(D3DPRESENT_PARAMETERS) );

    // Select back buffer format etc
	m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &CurrentMode);
	PresentParams.BackBufferFormat = CurrentMode.Format;
        
	// Setup remaining flags
	PresentParams.AutoDepthStencilFormat = FindDepthStencilFormat( D3DADAPTER_DEFAULT, CurrentMode, D3DDEVTYPE_HAL );
    PresentParams.SwapEffect			 = D3DSWAPEFFECT_DISCARD;
    PresentParams.PresentationInterval   = D3DPRESENT_INTERVAL_IMMEDIATE;
	PresentParams.Windowed				 = true;
	PresentParams.EnableAutoDepthStencil = true;
	PresentParams.BackBufferWidth = rc.Width();
	PresentParams.BackBufferHeight = rc.Height();
	// Set Creation Flags
	unsigned long ulFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

    // Check if Hardware T&L is available
    ZeroMemory( &Caps, sizeof(D3DCAPS9) );
    m_pD3D->GetDeviceCaps( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &Caps );
    if ( Caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT ) ulFlags = D3DCREATE_HARDWARE_VERTEXPROCESSING;
        
    // Attempt to create a HAL device
    if( FAILED( hRet = m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd, ulFlags, &PresentParams, &m_pD3DDevice ) ) ) 
    {
        MessageBox( m_hWnd, _T("Could not create a valid HAL Direct3D device object.\r\n\r\n")
                            _T("The system will now attempt to create a device utilising the 'Reference Rasterizer' (D3DDEVTYPE_REF)"),
                            _T("Fatal Error!"), MB_OK | MB_ICONINFORMATION | MB_APPLMODAL );
        
        // Find REF depth buffer format
        PresentParams.AutoDepthStencilFormat = FindDepthStencilFormat( D3DADAPTER_DEFAULT, CurrentMode, D3DDEVTYPE_REF );

        // Check if Hardware T&L is available
        ZeroMemory( &Caps, sizeof(D3DCAPS9) );
        ulFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;
        m_pD3D->GetDeviceCaps( D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, &Caps );
        if ( Caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT ) ulFlags = D3DCREATE_HARDWARE_VERTEXPROCESSING;

        // Attempt to create a REF device
        if( FAILED( hRet = m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, m_hWnd, ulFlags, &PresentParams, &m_pD3DDevice ) ) ) 
        {
            MessageBox( m_hWnd, _T("Could not create a valid REF Direct3D device object.\r\n\r\nThe system will now exit."),
                                _T("Fatal Error!"), MB_OK | MB_ICONSTOP | MB_APPLMODAL );

            // Failed
            return false;
        
        } // End if Failure (REF)

    } // End if Failure (HAL)

    // Store the present parameters
    m_D3DPresentParams = PresentParams;

    // Success!!
    return true;
}

//-----------------------------------------------------------------------------
// Name : FindDepthStencilFormat ()
// Desc : This function simply determines the best depth format that is
//        available for the specified mode.
// Note : No tests for stencil active depth buffers are made.
//-----------------------------------------------------------------------------
D3DFORMAT CGameApp::FindDepthStencilFormat( ULONG AdapterOrdinal, D3DDISPLAYMODE Mode, D3DDEVTYPE DevType )
{

    // Test for 24 bith depth buffer
    if (SUCCEEDED( m_pD3D->CheckDeviceFormat(AdapterOrdinal, DevType, Mode.Format, D3DUSAGE_DEPTHSTENCIL , D3DRTYPE_SURFACE , D3DFMT_D32 )))
    {
        if (SUCCEEDED( m_pD3D->CheckDepthStencilMatch ( AdapterOrdinal, DevType, Mode.Format, Mode.Format, D3DFMT_D32 ))) return D3DFMT_D32;
    
    } // End if 32bpp Available

    // Test for 24 bit depth buffer
    if (SUCCEEDED( m_pD3D->CheckDeviceFormat(AdapterOrdinal, DevType, Mode.Format, D3DUSAGE_DEPTHSTENCIL , D3DRTYPE_SURFACE , D3DFMT_D24X8 )))
    {
        if (SUCCEEDED( m_pD3D->CheckDepthStencilMatch ( AdapterOrdinal, DevType, Mode.Format, Mode.Format, D3DFMT_D24X8 ))) return D3DFMT_D24X8;
    
    } // End if 24bpp Available

    // Test for 16 bit depth buffer
    if (SUCCEEDED( m_pD3D->CheckDeviceFormat(AdapterOrdinal, DevType, Mode.Format, D3DUSAGE_DEPTHSTENCIL , D3DRTYPE_SURFACE , D3DFMT_D16 )))
    {
        if (SUCCEEDED( m_pD3D->CheckDepthStencilMatch ( AdapterOrdinal, DevType, Mode.Format, Mode.Format, D3DFMT_D16 ))) return D3DFMT_D16;
    
    } // End if 16bpp Available

    // No depth buffer supported
    return D3DFMT_UNKNOWN;

}

//-----------------------------------------------------------------------------
// Name : SetupGameState ()
// Desc : Sets up all the initial states required by the game.
//-----------------------------------------------------------------------------
void CGameApp::SetupGameState()
{
    // Generate an identity matrix
    D3DXMatrixIdentity( &m_mtxIdentity );

    // App is active
    m_bActive = true;

    // Setup the players CCamera, and extract the pointer.
    // This pointer will only ever become invalid on subsequent
    // calls to CPlayer::SetCameraMode and on player destruction.
    m_Player.SetCameraMode( CCamera::MODE_FPS );
    m_pCamera = m_Player.GetCamera();

    // Setup our player's default details
    m_Player.SetFriction( 250.0f ); // Per Second
    m_Player.SetGravity( D3DXVECTOR3( 0, -500.0f, 0 ) );
    m_Player.SetMaxVelocityXZ( 125.0f );
    m_Player.SetMaxVelocityY ( 400.0f );
    m_Player.SetCamOffset( D3DXVECTOR3( 0.0f, 10.0f, 0.0f ) );
    m_Player.SetCamLag( 0.0f );
    
    // Set up the players collision volume info
    VOLUME_INFO Volume;
    Volume.Min = D3DXVECTOR3( -3,  -2, -3 );
    Volume.Max = D3DXVECTOR3(  3,  10,  3 );
    m_Player.SetVolumeInfo( Volume );

    // Setup our cameras view details
    m_pCamera->SetFOV( 80.0f );
    m_pCamera->SetViewport( m_nViewX, m_nViewY, m_nViewWidth, m_nViewHeight, 1.01f, 5000.0f );

    // Set the CCamera volume info (matches player volume)
    m_pCamera->SetVolumeInfo( Volume );

    // Add the update callbacks required
    m_Player.AddPlayerCallback( CTerrain::UpdatePlayer, (LPVOID)&m_Terrain );
    m_Player.AddCameraCallback( CTerrain::UpdateCamera, (LPVOID)&m_Terrain );
    
    // Lets give a small initial rotation and set initial position
    m_Player.SetPosition( D3DXVECTOR3( 430.0f, 400.0f, 330.0f ) );
    m_Player.Rotate( 25, 45, 0 );
}

//-----------------------------------------------------------------------------
// Name : SetupRenderStates ()
// Desc : Sets up all the initial states required by the renderer.
//-----------------------------------------------------------------------------
void CGameApp::SetupRenderStates()
{
    // Test the device capabilities.
//    if (!TestDeviceCaps( )) { PostQuitMessage(0); return; }

    // Setup our D3D Device initial states
    m_pD3DDevice->SetRenderState( D3DRS_ZENABLE, D3DZB_TRUE );
    m_pD3DDevice->SetRenderState( D3DRS_DITHERENABLE,  TRUE );
    m_pD3DDevice->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_GOURAUD );
    m_pD3DDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
    m_pD3DDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

    // Set up sampler states.
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_MINFILTER    , m_MinFilter );
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER    , m_MagFilter );
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER    , m_MipFilter );
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_MAXANISOTROPY, m_Anisotropy );

    // Set texture / addressing / color ops
    m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    m_pD3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP  , D3DTOP_SELECTARG1 );
    m_pD3DDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );

    // Inform the terrain of how it should render
    m_Terrain.SetRenderMode( m_bSinglePass );

    // If we are performing single pass rendering
    if ( m_bSinglePass )
    {
        m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLORARG1, D3DTA_CURRENT );
        m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLORARG2, D3DTA_TEXTURE );
        m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP  , m_ColorOp );
        m_pD3DDevice->SetTextureStageState( 1, D3DTSS_TEXCOORDINDEX, 1 );
        m_pD3DDevice->SetSamplerState( 1, D3DSAMP_MINFILTER    , m_MinFilter );
        m_pD3DDevice->SetSamplerState( 1, D3DSAMP_MAGFILTER    , m_MagFilter );
        m_pD3DDevice->SetSamplerState( 1, D3DSAMP_MIPFILTER    , m_MipFilter );
        m_pD3DDevice->SetSamplerState( 1, D3DSAMP_MAXANISOTROPY, m_Anisotropy );
    
    } // End if Single-Pass
    else
    {
        // Disable second stage processing
        m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
        m_pD3DDevice->SetTexture( 1, NULL );
    
    } // End if Multi-Pass
    
    // Setup option dependant states
    m_pD3DDevice->SetRenderState( D3DRS_FILLMODE, m_FillMode );                

    // Setup our default vertex FVF code
    m_pD3DDevice->SetFVF( VERTEX_FVF );

// Next Light

    // Store new viewport sizes
    RECT rc;
    ::GetClientRect( m_hWnd, &rc );
    m_nViewWidth  = rc.right - rc.left;
    m_nViewHeight = rc.bottom - rc.top;

    // Update our device with our CCamera details (Required on reset)
    if ( !m_pCamera ) return;
    m_pCamera->SetViewport( m_nViewX, m_nViewY, m_nViewWidth, m_nViewHeight, 1.01f, 5000.0f );
    m_pCamera->UpdateRenderView( m_pD3DDevice );
    m_pCamera->UpdateRenderProj( m_pD3DDevice );

    // Set up the menu item selections (Which may have changed during device validations)
//    SelectMenuItems();
}

//-----------------------------------------------------------------------------
// Name : ShutDown ()
// Desc : Shuts down the game engine, and frees up all resources.
//-----------------------------------------------------------------------------
bool CGameApp::ShutDown()
{
	 // Release any previously built objects
    ReleaseObjects ( );

    // Destroy Direct3D Objects
    if ( m_pD3DDevice ) m_pD3DDevice->Release();
    if ( m_pD3D       ) m_pD3D->Release();
    m_pD3D          = NULL;
    m_pD3DDevice    = NULL;
    
    // Destroy menu, it may not be attached
    if ( m_hMenu ) DestroyMenu( m_hMenu );
    m_hMenu         = NULL;

    // Destroy the render window
    SetMenu( m_hWnd, NULL );
    if ( m_hWnd ) DestroyWindow( m_hWnd );
    m_hWnd          = NULL;
    
    // Shutdown Success
    return true;
}

//-----------------------------------------------------------------------------
// Name : BuildObjects ()
// Desc : Build our demonstration cube mesh, and the objects that instance it
//-----------------------------------------------------------------------------
bool CGameApp::BuildObjects()
{
 //   VERTEXPROCESSING_TYPE vp = m_D3DSettings.GetSettings()->VertexProcessingType;
    bool HardwareTnL = true;
    
    // Are we using HardwareTnL ?
//    if ( vp != HARDWARE_VP && vp != PURE_HARDWARE_VP ) HardwareTnL = false;

    // Release previously built objects
    ReleaseObjects();

    // Build our terrain data
    m_Terrain.SetD3DDevice( m_pD3DDevice, HardwareTnL, &m_Timer);
    if ( !m_Terrain.LoadHeightMap( L"Data/Map/HeightMap.raw", 257, 257 )) return false;
	
	// ٴ .. ũ ŭ..
	if( !BuildOcean(257,257))
		return false;

	// Ȱ 
	if(!m_Fog.Init(m_pD3DDevice))
		return false;

    // Build a 'player' mesh (this is just a cube currently)
    CLitVertex * pVertex = NULL;
    srand( timeGetTime() );

    // Add the 8 cube vertices to this mesh
    m_PlayerMesh.SetVertexFormat( LITVERTEX_FVF, sizeof(CLitVertex));
    if ( m_PlayerMesh.AddVertex( 8 ) < 0 ) return false;
    pVertex = (CLitVertex*)m_PlayerMesh.m_pVertex;
    
    // Add bottom 4 vertices
    *pVertex++ = CLitVertex( -3, 0, -3, RANDOM_COLOR );
    *pVertex++ = CLitVertex( -3, 0,  3, RANDOM_COLOR );
    *pVertex++ = CLitVertex(  3, 0,  3, RANDOM_COLOR );
    *pVertex++ = CLitVertex(  3, 0, -3, RANDOM_COLOR );
    
    // Add top 4 vertices
    *pVertex++ = CLitVertex( -3, 20, -3, RANDOM_COLOR );
    *pVertex++ = CLitVertex( -3, 20,  3, RANDOM_COLOR );
    *pVertex++ = CLitVertex(  3, 20,  3, RANDOM_COLOR );
    *pVertex++ = CLitVertex(  3, 20, -3, RANDOM_COLOR );

    // Add the indices as a strip (with one degenerate) ;)
    if ( m_PlayerMesh.AddIndex( 16 ) < 0 ) return false;
    m_PlayerMesh.m_pIndex[ 0] = 5;
    m_PlayerMesh.m_pIndex[ 1] = 6;
    m_PlayerMesh.m_pIndex[ 2] = 4;
    m_PlayerMesh.m_pIndex[ 3] = 7; //               5-6
    m_PlayerMesh.m_pIndex[ 4] = 0; //               |/|
    m_PlayerMesh.m_pIndex[ 5] = 3; //               4-7
    m_PlayerMesh.m_pIndex[ 6] = 1; //               |/|
    m_PlayerMesh.m_pIndex[ 7] = 2; //               0-3-7
    m_PlayerMesh.m_pIndex[ 8] = 3; // Degen Index   |/|/|
    m_PlayerMesh.m_pIndex[ 9] = 7; //               1-2-6
    m_PlayerMesh.m_pIndex[10] = 2; //                 |/|
    m_PlayerMesh.m_pIndex[11] = 6; //                 1-5
    m_PlayerMesh.m_pIndex[12] = 1; //                 |/|
    m_PlayerMesh.m_pIndex[13] = 5; //                 0-4
    m_PlayerMesh.m_pIndex[14] = 0;
    m_PlayerMesh.m_pIndex[15] = 4;

    // Build the mesh's vertex and index buffers
    if (FAILED(m_PlayerMesh.BuildBuffers( m_pD3DDevice, HardwareTnL, true ))) return false;

    // Our object references this mesh
    m_Object.m_pMesh = &m_PlayerMesh;

    // Link this object to our player
    m_Player.Set3rdPersonObject( &m_Object );

    // Build the skybox
    if ( !BuildSkyBox() ) return false;

    // Success!
    return true;
}

//-----------------------------------------------------------------------------
// Name : BuildSkyBox ()
// Desc : Build the sky box data ready for rendering.
//-----------------------------------------------------------------------------
bool CGameApp::BuildSkyBox()
{
    HRESULT hRet;
    
    // Allocate our 24 mesh vertices
    m_SkyMesh.SetVertexFormat( VERTEX_FVF, sizeof(CVertex) );
    if ( m_SkyMesh.AddVertex( 24 ) < 0 ) return false;
    
    // Build the skybox vertices
    CVertex * pVertices = (CVertex*)m_SkyMesh.m_pVertex;

    // Front quad (remember all quads point inward)
    pVertices[0]  = CVertex( -10.0f,  10.0f,  10.0f, 0.0f, 0.0f );
    pVertices[1]  = CVertex(  10.0f,  10.0f,  10.0f, 1.0f, 0.0f );
    pVertices[2]  = CVertex(  10.0f, -10.0f,  10.0f, 1.0f, 1.0f );
    pVertices[3]  = CVertex( -10.0f, -10.0f,  10.0f, 0.0f, 1.0f );

    // Back Quad
    pVertices[4]  = CVertex(  10.0f,  10.0f, -10.0f, 0.0f, 0.0f );
    pVertices[5]  = CVertex( -10.0f,  10.0f, -10.0f, 1.0f, 0.0f );
    pVertices[6]  = CVertex( -10.0f, -10.0f, -10.0f, 1.0f, 1.0f );
    pVertices[7]  = CVertex(  10.0f, -10.0f, -10.0f, 0.0f, 1.0f );

    // Left Quad
    pVertices[8]  = CVertex( -10.0f,  10.0f, -10.0f, 0.0f, 0.0f );
    pVertices[9]  = CVertex( -10.0f,  10.0f,  10.0f, 1.0f, 0.0f );
    pVertices[10] = CVertex( -10.0f, -10.0f,  10.0f, 1.0f, 1.0f );
    pVertices[11] = CVertex( -10.0f, -10.0f, -10.0f, 0.0f, 1.0f );

    // Right Quad
    pVertices[12] = CVertex(  10.0f,  10.0f,  10.0f, 0.0f, 0.0f );
    pVertices[13] = CVertex(  10.0f,  10.0f, -10.0f, 1.0f, 0.0f );
    pVertices[14] = CVertex(  10.0f, -10.0f, -10.0f, 1.0f, 1.0f );
    pVertices[15] = CVertex(  10.0f, -10.0f,  10.0f, 0.0f, 1.0f );

    // Top Quad
    pVertices[16] = CVertex( -10.0f,  10.0f, -10.0f, 0.0f, 0.0f );
    pVertices[17] = CVertex(  10.0f,  10.0f, -10.0f, 1.0f, 0.0f );
    pVertices[18] = CVertex(  10.0f,  10.0f,  10.0f, 1.0f, 1.0f );
    pVertices[19] = CVertex( -10.0f,  10.0f,  10.0f, 0.0f, 1.0f );

    // Bottom Quad
    pVertices[20] = CVertex( -10.0f, -10.0f,  10.0f, 0.0f, 0.0f );
    pVertices[21] = CVertex(  10.0f, -10.0f,  10.0f, 1.0f, 0.0f );
    pVertices[22] = CVertex(  10.0f, -10.0f, -10.0f, 1.0f, 1.0f );
    pVertices[23] = CVertex( -10.0f, -10.0f, -10.0f, 0.0f, 1.0f );

    // Allocate the indices
    if ( m_SkyMesh.AddIndex( 24 ) < 0 ) return false;
    USHORT * pIndices = m_SkyMesh.m_pIndex;

    // Set the indices for each face (tri strip)
    for ( USHORT i = 0; i < 6; i++ )
    {
        // Build the skybox indices
        *pIndices++ = (i*4);
        *pIndices++ = (i*4) + 1;
        *pIndices++ = (i*4) + 3;
        *pIndices++ = (i*4) + 2;
    
    } // Next 'face'

//    VERTEXPROCESSING_TYPE vp = m_D3DSettings.GetSettings()->VertexProcessingType;
    bool HardwareTnL = true;
    
    // Are we using HardwareTnL ?
 //   if ( vp != HARDWARE_VP && vp != PURE_HARDWARE_VP ) HardwareTnL = false;

    // Build the mesh buffers
    if ( FAILED(m_SkyMesh.BuildBuffers( m_pD3DDevice, HardwareTnL ))) return false;

    // Load Textures
	std::wstring name = L"Data/SkyBox/morning";
	std::wstring filename = name + L"_FR.jpg";
	hRet  = D3DXCreateTextureFromFile( m_pD3DDevice, filename.c_str(), &m_SkyTextures[0] );

	filename = name + L"_BK.jpg";
    hRet |= D3DXCreateTextureFromFile( m_pD3DDevice, filename.c_str(), &m_SkyTextures[1] );

	filename = name + L"_LF.jpg";
    hRet |= D3DXCreateTextureFromFile( m_pD3DDevice, filename.c_str(), &m_SkyTextures[2] );

	filename = name + L"_RT.jpg";
    hRet |= D3DXCreateTextureFromFile( m_pD3DDevice, filename.c_str(), &m_SkyTextures[3] );

	filename = name + L"_UP.jpg";
    hRet |= D3DXCreateTextureFromFile( m_pD3DDevice, filename.c_str(), &m_SkyTextures[4] );

	filename = name + L"_DN.jpg";
    hRet |= D3DXCreateTextureFromFile( m_pD3DDevice, filename.c_str(), &m_SkyTextures[5] );
    if ( FAILED(hRet) ) return false;
    
    // Success!!
    return true;
}

//-----------------------------------------------------------------------------
// Name : RenderSkyBox ()
// Desc : Render the sky box to the frame buffer.
//-----------------------------------------------------------------------------
void CGameApp::RenderSkyBox( )
{
    D3DXMATRIXA16 mtxWorld;
    D3DXMatrixIdentity( &mtxWorld );

    // Validate parameters
    if ( !m_pCamera || !m_pD3DDevice ) return;
    
    // Generate our sky box rendering origin and set as world matrix
    D3DXVECTOR3 CamPos = m_pCamera->GetPosition();
    D3DXMatrixTranslation( &mtxWorld, CamPos.x, CamPos.y + 1.3f, CamPos.z );
    m_pD3DDevice->SetTransform( D3DTS_WORLD, &mtxWorld );

    // Set up rendering states for the sky box
    m_pD3DDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
    m_pD3DDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );

    // Disable second stage if enabled
    if ( m_bSinglePass ) m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
    
    // Render the sky box
    m_pD3DDevice->SetFVF( m_SkyMesh.m_nFVFCode );
    m_pD3DDevice->SetIndices( m_SkyMesh.m_pIndexBuffer );
    m_pD3DDevice->SetStreamSource( 0, m_SkyMesh.m_pVertexBuffer, 0, m_SkyMesh.m_nStride );
    
    // Render the 6 sides of the skybox
    for ( ULONG i = 0; i < 6; ++i )
    {
        m_pD3DDevice->SetTexture( 0, m_SkyTextures[i] );
        m_pD3DDevice->DrawIndexedPrimitive( D3DPT_TRIANGLESTRIP, 0, 0, 24, i * 4, 2 );

    } // Next side

    // Reset our states
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
    m_pD3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
    m_pD3DDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
    m_pD3DDevice->SetTransform( D3DTS_WORLD, &m_mtxIdentity );

    // Re-enable second stage if enabled
    if ( m_bSinglePass ) m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP, m_ColorOp );
}

bool CGameApp::BuildOcean(float Width, float Height)
{
	const LPWSTR WaterNormalMapName  = L"Data/Water/wave2.dds";
	const LPWSTR WaterTextureName  = L"Data/Water/Water_Texture.jpg";

	// First of all store the information passed
 
    // A scale of 4 is roughly the best size for a 512 x 512 quad terrain.
    // Using the following forumla, lowering the size of the terrain 
    // simply lowers the vertex resolution but maintains the map size.
  	
    float x = Width * 4.0f * (512 / (Width - 1));
	float y = Main_Core::WaterLevel * 2.0f;
	float z = Height * 4.0f * (512 / (Height - 1));

	if( !m_Ocean.Init(m_pD3DDevice, D3DXVECTOR3(0,y,0), x, z, WaterNormalMapName, &m_SkyTextures[4]))
		return false;

	return true;
}

void CGameApp::RenderOcean()
{
	float WaterHeight = Main_Core::WaterLevel * 2.0f;
    // Disable second texture stage if in use
    if ( m_bSinglePass )
	{
		m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP, D3DTOP_DISABLE );
	}
		
	//  ̴ ... ī޶  ̸ true  ȯǼ camera Ưȿ..
	if(m_Ocean.Rander(m_pCamera->GetPosition()))
	{
		m_pCamera->RenderScreenEffect( m_pD3DDevice, CCamera::EFFECT_WATER, *(ULONG*)(&WaterHeight), 
									D3DCOLOR_COLORVALUE(0.0f, 0.03f, 0.05f, 0.4f));
	}	
	 
	// Re-enable second texture stage if required
    if ( m_bSinglePass ) 
	{
		m_pD3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP, g_App.GetColorOp() );
	}
}

//-----------------------------------------------------------------------------
// Name : FrameAdvance () (Private)
// Desc : Called to signal that we are now rendering the next frame.
//-----------------------------------------------------------------------------
void CGameApp::FrameAdvance()
{
    static TCHAR FrameRate[ 50 ];
    static TCHAR TitleBuffer[ 255 ];

    // Advance the timer
    m_Timer.Tick( );

    // Skip if app is inactive
    if ( !m_bActive ) return;
    
    // Get / Display the framerate
    if ( m_LastFrameRate != m_Timer.GetFrameRate() )
    {
		CWnd *pWnd = AfxGetMainWnd();
		HWND hwnd = pWnd->m_hWnd;

        m_LastFrameRate = m_Timer.GetFrameRate( FrameRate );
		_stprintf( TitleBuffer, L"Creater [Kim Dong Sung] : 3D MapTool project <FPS : %s>", FrameRate );
        SetWindowText( hwnd, TitleBuffer );

    } // End if Frame Rate Altered
	SelectMenuItems();

    // Poll & Process input devices
    ProcessInput();

    // Animate the game objects
    AnimateObjects();

    // Clear the frame & depth buffer ready for drawing
    m_pD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
                         D3DCOLOR_COLORVALUE(0.0f,0.0f,1.0f,1.0f), 1.0f, 0 );
    
    // Begin Scene Rendering
    m_pD3DDevice->BeginScene();
	{
		// Skybox must ALWAYS be rendered before anything
		RenderSkyBox();
	    
		// Render player before terrain, because terrain may render alpha components
		m_Player.Render( m_pD3DDevice );

		// Reset our world matrix (player sets it)
		m_pD3DDevice->SetTransform( D3DTS_WORLD, &m_mtxIdentity );
		
		if(g_Option._Fog._using)
		{
			if(g_Option._Fog.style == Fog::Linear)
				m_Fog.Rander_Begin_Exp2(g_Option._Fog.exp2, g_Option._Fog.color);
			if(g_Option._Fog.style == Fog::Exp2)
				m_Fog.Rander_Begin_LINEAR(g_Option._Fog.fog_start, g_Option._Fog.fog_end, g_Option._Fog.color);
		}
			// Render our terrain objects
		m_Terrain.Render( m_pCamera );
		
		if(g_Option._Fog._using)
		{
			m_Fog.Rander_End();
		}
		
		if(g_Option._Ocean._using)
		{
			RenderOcean();
		}
	}
	// End Scene Rendering
    m_pD3DDevice->EndScene();
    
    // Present the buffer
    if ( FAILED(m_pD3DDevice->Present( NULL, NULL, NULL, NULL )) ) m_bLostDevice = true;

}

//-----------------------------------------------------------------------------
// Name : ProcessInput () (Private)
// Desc : Simply polls the input devices and performs basic input operations
//-----------------------------------------------------------------------------
void CGameApp::ProcessInput( )
{
    static UCHAR pKeyBuffer[ 256 ];
    ULONG        Direction = 0;
    POINT        CursorPos;
    float        X = 0.0f, Y = 0.0f;

    // Retrieve keyboard state
    if ( !GetKeyboardState( pKeyBuffer ) ) return;

    // Check the relevant keys
    if ( pKeyBuffer[ VK_UP    ] & 0xF0 ) Direction |= CPlayer::DIR_FORWARD;
    if ( pKeyBuffer[ VK_DOWN  ] & 0xF0 ) Direction |= CPlayer::DIR_BACKWARD;
    if ( pKeyBuffer[ VK_LEFT  ] & 0xF0 ) Direction |= CPlayer::DIR_LEFT;
    if ( pKeyBuffer[ VK_RIGHT ] & 0xF0 ) Direction |= CPlayer::DIR_RIGHT;
    if ( pKeyBuffer[ VK_PRIOR ] & 0xF0 ) Direction |= CPlayer::DIR_UP;
    if ( pKeyBuffer[ VK_NEXT  ] & 0xF0 ) Direction |= CPlayer::DIR_DOWN;
    
    // Now process the mouse (if the button is pressed)
    if ( GetCapture() == m_hWnd )
    {
        // Hide the mouse pointer
        SetCursor( NULL );

        // Retrieve the cursor position
        GetCursorPos( &CursorPos );

        // Calculate mouse rotational values
        X = (float)(CursorPos.x - m_OldCursorPos.x) / 3.0f;
        Y = (float)(CursorPos.y - m_OldCursorPos.y) / 3.0f;

        // Reset our cursor position so we can keep going forever :)
        SetCursorPos( m_OldCursorPos.x, m_OldCursorPos.y );

    } // End if Captured

    // Update if we have moved
    if ( Direction > 0 || X != 0.0f || Y != 0.0f )
    {
        // Rotate our CCamera
        if ( X || Y ) 
        {
            // Are they holding the right mouse button ?
            if ( pKeyBuffer[ VK_RBUTTON ] & 0xF0 )
                m_Player.Rotate( Y, 0.0f, -X );
            else
                m_Player.Rotate( Y, X, 0.0f );
        
        } // End if any rotation

        // Any Movement ?
        if ( Direction ) 
        {
            // Move our player (Force applied must be greater than total friction)
            m_Player.Move( Direction, 500.0f * m_Timer.GetTimeElapsed(), true );

        } // End if any movement

    } // End if CCamera moved

    // Update our CCamera (updates velocity etc)
    m_Player.Update( m_Timer.GetTimeElapsed() );

    // Update the device matrix
    m_pCamera->UpdateRenderView( m_pD3DDevice );

}

//-----------------------------------------------------------------------------
// Name : AnimateObjects () (Private)
// Desc : Animates the objects we currently have loaded.
//-----------------------------------------------------------------------------
void CGameApp::AnimateObjects()
{
    // Note : Expanded for the purposes of this example only.
}

//-----------------------------------------------------------------------------
// Name : ReleaseObjects ()
// Desc : Releases our objects and their associated memory so that we can
//        rebuild them, if required, during our applications life-time.
//-----------------------------------------------------------------------------
void CGameApp::ReleaseObjects( )
{
    // De-reference device objects stream (just in case any remain set)
    if ( m_pD3DDevice ) 
    {
        m_pD3DDevice->SetStreamSource( 0, NULL, 0, 0 );
        m_pD3DDevice->SetIndices( NULL );
        m_pD3DDevice->SetTexture( 0, NULL );
        m_pD3DDevice->SetTexture( 1, NULL );

    } // End if Device Available

    // Release textures
    for ( ULONG i = 0; i < 6; ++i ) if ( m_SkyTextures[i] ) m_SkyTextures[i]->Release();
    ZeroMemory( m_SkyTextures, 6 * sizeof(LPDIRECT3DTEXTURE9) );

    // Release any required objects
    m_Terrain.Release();
}

//-----------------------------------------------------------------------------
// MFC  ϱ  Լ
//-----------------------------------------------------------------------------
void CGameApp::Mouse_Lift_Down()
{
	// Capture the mouse
	SetCapture( m_hWnd );
	GetCursorPos( &m_OldCursorPos );

}

void CGameApp::Mouse_Lift_Up()
{
	ReleaseCapture( );
}

void CGameApp::SetSize(bool isMin, CRect rc)
{
	if (isMin)
	{
		// App is inactive
		m_bActive = false;

	} // App has been minimized
	else
	{
		// App is active
		m_bActive = true;

		// Store new viewport sizes
		m_nViewWidth  = rc.Width();
		m_nViewHeight = rc.Height();

		if (m_pD3DDevice) 
		{
			// Store new sizes
			m_D3DPresentParams.BackBufferWidth = m_nViewWidth;
			m_D3DPresentParams.BackBufferHeight = m_nViewHeight;

			// Reset the device
			m_pD3DDevice->Reset( &m_D3DPresentParams );
			SetupRenderStates( );

		} // End if

	} // End if !Minimized

}

void CGameApp::Camera_Mode_Change(int camera_num)
{
	m_CameraMode = camera_num;

	switch(m_CameraMode)
	{
	//	case CCamera::MODE_FPS:
		case Play::_Camera::FPS:
			// Set CCamera mode to FPS style
			// Setup player details
			m_Player.SetFriction      ( 250.0f ); // Per Second
			m_Player.SetGravity       ( D3DXVECTOR3( 0, -400.0f, 0 ) );
			m_Player.SetMaxVelocityXZ ( 125.0f );
			m_Player.SetMaxVelocityY  ( 400.0f );
			m_Player.SetCamOffset     ( D3DXVECTOR3( 0.0f, 10.0f, 0.0f ) );
			m_Player.SetCamLag        ( 0.0f ); // No CCamera lag

			// Switch CCamera mode
			m_Player.SetCameraMode    ( CCamera::MODE_FPS );
			m_pCamera = m_Player.GetCamera();

			break;

	//	case CCamera::MODE_SPACECRAFT:
		case Play::_Camera::CRAFT:
			// Set CCamera mode to SPACECRAFT style
			// Setup player details
			m_Player.SetFriction      ( 125.0f ); // Per Second
			m_Player.SetGravity       ( D3DXVECTOR3( 0, 0, 0 ) );
			m_Player.SetMaxVelocityXZ ( 400.0f );
			m_Player.SetMaxVelocityY  ( 400.0f );
			m_Player.SetCamOffset     ( D3DXVECTOR3( 0.0f, 0.0f, 0.0f ) );
			m_Player.SetCamLag        ( 0.0f ); // No CCamera lag

			// Switch CCamera mode
			m_Player.SetCameraMode( CCamera::MODE_SPACECRAFT );
			m_pCamera = m_Player.GetCamera();

			break;

	//	case CCamera::MODE_THIRDPERSON:
		case Play::_Camera::MMORPG:
			// Set CCamera mode to third person style
			// Setup Player details
			m_Player.SetFriction      ( 250.0f ); // Per Second
			m_Player.SetGravity       ( D3DXVECTOR3( 0, -400.0f, 0 ) );
			m_Player.SetMaxVelocityXZ ( 125.0f );
			m_Player.SetMaxVelocityY  ( 400.0f );
			m_Player.SetCamOffset     ( D3DXVECTOR3( 0.0f, 40.0f, -60.0f ) );
			m_Player.SetCamLag        ( 0.25f ); // 1/4 second CCamera lag

			// Switch CCamera mode
			m_Player.SetCameraMode    ( CCamera::MODE_THIRDPERSON );
			m_pCamera = m_Player.GetCamera();
			break;
		}
}

void CGameApp::Change_Direct3D()
{
	m_pD3DDevice->SetRenderState( D3DRS_FILLMODE, m_FillMode );
	SetupRenderStates();
}

void CGameApp::Change_Terrian(bool SinglePass)
{
	m_bSinglePass = SinglePass;
    m_Terrain.SetRenderMode( m_bSinglePass );
	SetupRenderStates();
}

//-----------------------------------------------------------------------------
// Name : SelectMenuItems ()
// Desc : Sets up our application's menu item selection states.
//-----------------------------------------------------------------------------
void CGameApp::SelectMenuItems()
{
	m_FillMode		= g_Option._Play.fillmode;
	if(m_FillMode == D3DFILLMODE::D3DFILL_SOLID)
	{
		m_pD3DDevice->SetRenderState( D3DRS_FILLMODE, m_FillMode );
		//
	}
	m_ColorOp		= g_Option._Play.color;			// Which color op are we using?
	m_MagFilter		= g_Option._Play.mag;			// Magnification Filter to use.
	m_MinFilter		= g_Option._Play.pow;			// Minification Filter to use.
	m_MipFilter		= g_Option._Play.mip;			// Mip-Map filter to use.
	m_Anisotropy	= (ULONG)g_Option._Play.aniso;	// Anisotropy level to use.
	m_bSinglePass	= g_Option._Play.pass;			// Use single pass rendering method (multiple stage)

	Camera_Mode_Change(g_Option._Play.camera);

	SetupRenderStates();	
}